@@ -20,8 +20,9 @@
 
 #include "calendarview_debug.h"
 
-#include <Akonadi/Calendar/ETMCalendar>
-#include <Akonadi/Calendar/IncidenceChanger>
+#include <Akonadi/CalendarUtils>
+#include <Akonadi/ETMCalendar>
+#include <Akonadi/IncidenceChanger>
 #include <CalendarSupport/CollectionSelection>
 #include <CalendarSupport/KCalPrefs>
 #include <CalendarSupport/Utils>
@@ -32,7 +33,7 @@
 
 #include <KIconLoader> // for SmallIcon()
 #include <KMessageBox>
-#include <KServiceTypeTrader>
+#include <KPluginFactory>
 #include <KSqueezedTextLabel>
 
 #include <KLocalizedString>
@@ -59,7 +60,7 @@ enum {
 };
 
 // Layout which places the widgets in equally sized columns,
-// matching the calculcation of the columns in the agenda.
+// matching the calculation of the columns in the agenda.
 class AgendaHeaderLayout : public QLayout
 {
 public:
@@ -421,33 +422,15 @@ void AgendaHeader::loadDecorations(const QStringList &decorations, const QString
 
 CalendarDecoration::Decoration *AgendaHeader::loadCalendarDecoration(const QString &name)
 {
-    const QString type = CalendarSupport::Plugin::serviceType();
-    const int version = CalendarSupport::Plugin::interfaceVersion();
+    auto result = KPluginFactory::instantiatePlugin<CalendarDecoration::Decoration>(
+        KPluginMetaData(QStringLiteral("pim" QT_STRINGIFY(QT_VERSION_MAJOR)) + QStringLiteral("/korganizer/") + name));
 
-    QString constraint;
-    if (version >= 0) {
-        constraint = QStringLiteral("[X-KDE-PluginInterfaceVersion] == %1").arg(QString::number(version));
+    if (result) {
+        return result.plugin;
+    } else {
+        qCDebug(CALENDARVIEW_LOG) << "Factory creation failed" << result.errorString;
+        return nullptr;
     }
-
-    KService::List list = KServiceTypeTrader::self()->query(type, constraint);
-
-    KService::List::ConstIterator it;
-    for (it = list.constBegin(); it != list.constEnd(); ++it) {
-        if ((*it)->desktopEntryName() == name) {
-            KService::Ptr service = *it;
-            KPluginLoader loader(*service);
-
-            auto factory = loader.factory();
-            if (!factory) {
-                qCDebug(CALENDARVIEW_LOG) << "Factory creation failed";
-                return nullptr;
-            }
-
-            return factory->create<CalendarDecoration::Decoration>();
-        }
-    }
-
-    return nullptr;
 }
 
 class EventViews::EventIndicatorPrivate
@@ -1276,7 +1259,7 @@ void AgendaView::connectAgenda(Agenda *agenda, Agenda *otherAgenda)
 
     agenda->setCalendar(d->mViewCalendar);
 
-    connect(agenda, SIGNAL(newEventSignal()), SIGNAL(newEventSignal()));
+    connect(agenda, &Agenda::newEventSignal, this, qOverload<>(&EventView::newEventSignal));
 
     connect(agenda, &Agenda::newStartSelectSignal, otherAgenda, &Agenda::clearSelection);
     connect(agenda, &Agenda::newStartSelectSignal, this, &AgendaView::timeSpanSelectionChanged);
@@ -1806,7 +1789,7 @@ void AgendaView::updateEventDates(AgendaItem *item, bool addIncidence, Akonadi::
         Akonadi::Collection collection = calendar()->collection(collectionId);
         result = changer()->createIncidence(incidence, collection, this) != -1;
     } else {
-        KCalendarCore::Incidence::Ptr oldIncidence(CalendarSupport::incidence(aitem));
+        KCalendarCore::Incidence::Ptr oldIncidence(Akonadi::CalendarUtils::incidence(aitem));
         aitem.setPayload<KCalendarCore::Incidence::Ptr>(incidence);
         result = changer()->modifyIncidence(aitem, oldIncidence, this) != -1;
     }
@@ -1885,7 +1868,7 @@ void AgendaView::showIncidences(const Akonadi::Item::List &incidences, const QDa
     bool wehaveall = true;
     if (filter) {
         for (const Akonadi::Item &aitem : incidences) {
-            if (!(wehaveall = filter->filterIncidence(CalendarSupport::incidence(aitem)))) {
+            if (!(wehaveall = filter->filterIncidence(Akonadi::CalendarUtils::incidence(aitem)))) {
                 break;
             }
         }
@@ -1895,15 +1878,15 @@ void AgendaView::showIncidences(const Akonadi::Item::List &incidences, const QDa
         calendar()->setFilter(nullptr);
     }
 
-    QDateTime start = CalendarSupport::incidence(incidences.first())->dtStart().toLocalTime();
-    QDateTime end = CalendarSupport::incidence(incidences.first())->dateTime(KCalendarCore::Incidence::RoleEnd).toLocalTime();
+    QDateTime start = Akonadi::CalendarUtils::incidence(incidences.first())->dtStart().toLocalTime();
+    QDateTime end = Akonadi::CalendarUtils::incidence(incidences.first())->dateTime(KCalendarCore::Incidence::RoleEnd).toLocalTime();
     Akonadi::Item first = incidences.first();
     for (const Akonadi::Item &aitem : incidences) {
-        if (CalendarSupport::incidence(aitem)->dtStart().toLocalTime() < start) {
+        if (Akonadi::CalendarUtils::incidence(aitem)->dtStart().toLocalTime() < start) {
             first = aitem;
         }
-        start = qMin(start, CalendarSupport::incidence(aitem)->dtStart().toLocalTime());
-        end = qMax(start, CalendarSupport::incidence(aitem)->dateTime(KCalendarCore::Incidence::RoleEnd).toLocalTime());
+        start = qMin(start, Akonadi::CalendarUtils::incidence(aitem)->dtStart().toLocalTime());
+        end = qMax(start, Akonadi::CalendarUtils::incidence(aitem)->dateTime(KCalendarCore::Incidence::RoleEnd).toLocalTime());
     }
 
     end.toTimeZone(start.timeZone()); // allow direct comparison of dates
@@ -2150,11 +2133,11 @@ void AgendaView::slotIncidencesDropped(const QList<QUrl> &items, const QPoint &g
     KDateTime newTime(day, time, preferences()->timeSpec());
     newTime.setDateOnly(allDay);
 
-    Todo::Ptr todo = CalendarSupport::todo(todoItem);
+    Todo::Ptr todo = Akonadi::CalendarUtils4::todo(todoItem);
     if (todo && dynamic_cast<Akonadi::ETMCalendar *>(calendar())) {
         const Akonadi::Item existingTodoItem = calendar()->itemForIncidence(calendar()->todo(todo->uid()));
 
-        if (Todo::Ptr existingTodo = CalendarSupport::todo(existingTodoItem)) {
+        if (Todo::Ptr existingTodo = Akonadi::CalendarUtils::todo(existingTodoItem)) {
             qCDebug(CALENDARVIEW_LOG) << "Drop existing Todo";
             Todo::Ptr oldTodo(existingTodo->clone());
             if (changer()) {
@@ -2162,7 +2145,7 @@ void AgendaView::slotIncidencesDropped(const QList<QUrl> &items, const QPoint &g
                 existingTodo->setAllDay(allDay);
                 changer()->modifyIncidence(existingTodoItem, oldTodo, this);
             } else {
-                KMessageBox::sorry(this,
+                KMessageBox::error(this,
                                    i18n("Unable to modify this to-do, "
                                         "because it cannot be locked."));
             }
@@ -2171,7 +2154,7 @@ void AgendaView::slotIncidencesDropped(const QList<QUrl> &items, const QPoint &g
             todo->setDtDue(newTime);
             todo->setAllDay(allDay);
             if (!changer()->addIncidence(todo, this)) {
-                KMessageBox::sorry(this, i18n("Unable to save %1 \"%2\".", i18n(todo->type()), todo->summary()));
+                KMessageBox::error(this, i18n("Unable to save %1 \"%2\".", i18n(todo->type()), todo->summary()));
             }
         }
     }
@@ -2307,7 +2290,7 @@ void AgendaView::writeSettings(KConfig *config)
 QVector<bool> AgendaView::busyDayMask() const
 {
     if (d->mSelectedDates.isEmpty() || !d->mSelectedDates[0].isValid()) {
-        return QVector<bool>();
+        return {};
     }
 
     QVector<bool> busyDayMask;
