@@ -11,8 +11,8 @@
 
 #include "eventviews_export.h"
 
-#include <Akonadi/Calendar/ETMCalendar>
 #include <Akonadi/Collection>
+#include <Akonadi/ETMCalendar>
 #include <Akonadi/Item>
 
 #include <KCalendarCore/Incidence>
@@ -496,4 +496,3 @@ private:
     Q_DECLARE_PRIVATE(EventView)
 };
 }
-
