@@ -1,25 +1,9 @@
 /*
-  Copyright (c) 2008 Bruno Virlet <bruno.virlet@gmail.com>
-  Copyright (C) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.net
-  Author: Bertjan Broeksema, broeksema@kde.org
+  SPDX-FileCopyrightText: 2008 Bruno Virlet <bruno.virlet@gmail.com>
+  SPDX-FileCopyrightText: 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.net>
+  SPDX-FileContributor: Bertjan Broeksema <broeksema@kde.org>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-
-  As a special exception, permission is given to link this program
-  with any edition of Qt, and distribute the resulting executable,
-  without including the source code for Qt in the source distribution.
+  SPDX-License-Identifier: GPL-2.0-or-later WITH Qt-Commercial-exception-1.0
 */
 
 #include "monthview.h"
@@ -28,16 +12,16 @@
 #include "monthscene.h"
 #include "prefs.h"
 
+#include <Akonadi/ETMCalendar>
 #include <CalendarSupport/CollectionSelection>
-#include <Akonadi/Calendar/ETMCalendar>
 #include <CalendarSupport/KCalPrefs>
 #include <CalendarSupport/Utils>
 
+#include "calendarview_debug.h"
 #include <KCalendarCore/OccurrenceIterator>
 #include <KCheckableProxyModel>
 #include <KLocalizedString>
 #include <QIcon>
-#include "calendarview_debug.h"
 
 #include <QHBoxLayout>
 #include <QTimer>
@@ -46,10 +30,11 @@
 
 using namespace EventViews;
 
-namespace EventViews {
+namespace EventViews
+{
 class MonthViewPrivate : public Akonadi::ETMCalendar::CalendarObserver
 {
-    MonthView *q;
+    MonthView *const q;
 
 public: /// Methods
     explicit MonthViewPrivate(MonthView *qq);
@@ -59,7 +44,7 @@ public: /// Methods
     // void setUpModels();
     void triggerDelayedReload(EventView::Change reason);
 
-public:  /// Members
+public: /// Members
     QTimer reloadTimer;
     MonthScene *scene = nullptr;
     QDate selectedItemDate;
@@ -68,15 +53,16 @@ public:  /// Members
     QToolButton *fullView = nullptr;
 
     // List of uids for QDate
-    QMap<QDate, QStringList > mBusyDays;
+    QMap<QDate, QStringList> mBusyDays;
 
 protected:
     /* reimplemented from KCalendarCore::Calendar::CalendarObserver */
     void calendarIncidenceAdded(const KCalendarCore::Incidence::Ptr &incidence) override;
     void calendarIncidenceChanged(const KCalendarCore::Incidence::Ptr &incidence) override;
     void calendarIncidenceDeleted(const KCalendarCore::Incidence::Ptr &incidence, const KCalendarCore::Calendar *calendar) override;
+
 private:
-    //quiet --overloaded-virtual warning
+    // quiet --overloaded-virtual warning
     using KCalendarCore::Calendar::CalendarObserver::calendarIncidenceDeleted;
 };
 }
@@ -94,8 +80,8 @@ MonthViewPrivate::MonthViewPrivate(MonthView *qq)
 
 void MonthViewPrivate::addIncidence(const Akonadi::Item &incidence)
 {
-    Q_UNUSED(incidence);
-    //TODO: add some more intelligence here...
+    Q_UNUSED(incidence)
+    // TODO: add some more intelligence here...
     q->setChanges(q->changes() | EventView::IncidencesAdded);
     reloadTimer.start(50);
 }
@@ -148,7 +134,7 @@ void MonthViewPrivate::calendarIncidenceChanged(const KCalendarCore::Incidence::
 
 void MonthViewPrivate::calendarIncidenceDeleted(const KCalendarCore::Incidence::Ptr &incidence, const KCalendarCore::Calendar *calendar)
 {
-    Q_UNUSED(calendar);
+    Q_UNUSED(calendar)
     Q_ASSERT(!incidence->uid().isEmpty());
     scene->removeIncidence(incidence->uid());
 }
@@ -159,12 +145,12 @@ MonthView::MonthView(NavButtonsVisibility visibility, QWidget *parent)
     : EventView(parent)
     , d(new MonthViewPrivate(this))
 {
-    QHBoxLayout *topLayout = new QHBoxLayout(this);
+    auto topLayout = new QHBoxLayout(this);
     topLayout->addWidget(d->view);
     topLayout->setContentsMargins(0, 0, 0, 0);
 
     if (visibility == Visible) {
-        QVBoxLayout *rightLayout = new QVBoxLayout();
+        auto rightLayout = new QVBoxLayout();
         rightLayout->setSpacing(0);
         rightLayout->setContentsMargins(0, 0, 0, 0);
 
@@ -176,57 +162,40 @@ MonthView::MonthView(NavButtonsVisibility visibility, QWidget *parent)
         d->fullView->setAutoRaise(true);
         d->fullView->setCheckable(true);
         d->fullView->setChecked(preferences()->fullViewMonth());
-        d->fullView->isChecked()
-        ? d->fullView->setToolTip(i18nc("@info:tooltip",
-                                        "Display calendar in a normal size"))
-        : d->fullView->setToolTip(i18nc("@info:tooltip",
-                                        "Display calendar in a full window"));
-        d->fullView->setWhatsThis(
-            i18nc("@info:whatsthis",
-                  "Click this button and the month view will be enlarged to fill the "
-                  "maximum available window space / or shrunk back to its normal size."));
-        connect(d->fullView, &QAbstractButton::clicked,
-                this, &MonthView::changeFullView);
+        d->fullView->isChecked() ? d->fullView->setToolTip(i18nc("@info:tooltip", "Display calendar in a normal size"))
+                                 : d->fullView->setToolTip(i18nc("@info:tooltip", "Display calendar in a full window"));
+        d->fullView->setWhatsThis(i18nc("@info:whatsthis",
+                                        "Click this button and the month view will be enlarged to fill the "
+                                        "maximum available window space / or shrunk back to its normal size."));
+        connect(d->fullView, &QAbstractButton::clicked, this, &MonthView::changeFullView);
 
-        QToolButton *minusMonth = new QToolButton(this);
+        auto minusMonth = new QToolButton(this);
         minusMonth->setIcon(QIcon::fromTheme(QStringLiteral("arrow-up-double")));
         minusMonth->setAutoRaise(true);
         minusMonth->setToolTip(i18nc("@info:tooltip", "Go back one month"));
-        minusMonth->setWhatsThis(
-            i18nc("@info:whatsthis",
-                  "Click this button and the view will be scrolled back in time by 1 month."));
-        connect(minusMonth, &QAbstractButton::clicked,
-                this, &MonthView::moveBackMonth);
+        minusMonth->setWhatsThis(i18nc("@info:whatsthis", "Click this button and the view will be scrolled back in time by 1 month."));
+        connect(minusMonth, &QAbstractButton::clicked, this, &MonthView::moveBackMonth);
 
-        QToolButton *minusWeek = new QToolButton(this);
+        auto minusWeek = new QToolButton(this);
         minusWeek->setIcon(QIcon::fromTheme(QStringLiteral("arrow-up")));
         minusWeek->setAutoRaise(true);
         minusWeek->setToolTip(i18nc("@info:tooltip", "Go back one week"));
-        minusWeek->setWhatsThis(
-            i18nc("@info:whatsthis",
-                  "Click this button and the view will be scrolled back in time by 1 week."));
-        connect(minusWeek, &QAbstractButton::clicked,
-                this, &MonthView::moveBackWeek);
+        minusWeek->setWhatsThis(i18nc("@info:whatsthis", "Click this button and the view will be scrolled back in time by 1 week."));
+        connect(minusWeek, &QAbstractButton::clicked, this, &MonthView::moveBackWeek);
 
-        QToolButton *plusWeek = new QToolButton(this);
+        auto plusWeek = new QToolButton(this);
         plusWeek->setIcon(QIcon::fromTheme(QStringLiteral("arrow-down")));
         plusWeek->setAutoRaise(true);
         plusWeek->setToolTip(i18nc("@info:tooltip", "Go forward one week"));
-        plusWeek->setWhatsThis(
-            i18nc("@info:whatsthis",
-                  "Click this button and the view will be scrolled forward in time by 1 week."));
-        connect(plusWeek, &QAbstractButton::clicked,
-                this, &MonthView::moveFwdWeek);
+        plusWeek->setWhatsThis(i18nc("@info:whatsthis", "Click this button and the view will be scrolled forward in time by 1 week."));
+        connect(plusWeek, &QAbstractButton::clicked, this, &MonthView::moveFwdWeek);
 
-        QToolButton *plusMonth = new QToolButton(this);
+        auto plusMonth = new QToolButton(this);
         plusMonth->setIcon(QIcon::fromTheme(QStringLiteral("arrow-down-double")));
         plusMonth->setAutoRaise(true);
         plusMonth->setToolTip(i18nc("@info:tooltip", "Go forward one month"));
-        plusMonth->setWhatsThis(
-            i18nc("@info:whatsthis",
-                  "Click this button and the view will be scrolled forward in time by 1 month."));
-        connect(plusMonth, &QAbstractButton::clicked,
-                this, &MonthView::moveFwdMonth);
+        plusMonth->setWhatsThis(i18nc("@info:whatsthis", "Click this button and the view will be scrolled forward in time by 1 month."));
+        connect(plusMonth, &QAbstractButton::clicked, this, &MonthView::moveFwdMonth);
 
         rightLayout->addWidget(d->fullView);
         rightLayout->addWidget(minusMonth);
@@ -239,17 +208,13 @@ MonthView::MonthView(NavButtonsVisibility visibility, QWidget *parent)
         d->view->setFrameStyle(QFrame::NoFrame);
     }
 
-    connect(d->scene, &MonthScene::showIncidencePopupSignal,
-            this, &MonthView::showIncidencePopupSignal);
+    connect(d->scene, &MonthScene::showIncidencePopupSignal, this, &MonthView::showIncidencePopupSignal);
 
-    connect(d->scene, &MonthScene::incidenceSelected,
-            this, &EventView::incidenceSelected);
+    connect(d->scene, &MonthScene::incidenceSelected, this, &EventView::incidenceSelected);
 
-    connect(d->scene, SIGNAL(newEventSignal()),
-            SIGNAL(newEventSignal()));
+    connect(d->scene, &MonthScene::newEventSignal, this, qOverload<>(&EventView::newEventSignal));
 
-    connect(d->scene, &MonthScene::showNewEventPopupSignal,
-            this, &MonthView::showNewEventPopupSignal);
+    connect(d->scene, &MonthScene::showNewEventPopupSignal, this, &MonthView::showNewEventPopupSignal);
 
     connect(&d->reloadTimer, &QTimer::timeout, this, &MonthView::reloadIncidences);
     updateConfig();
@@ -261,10 +226,8 @@ MonthView::MonthView(NavButtonsVisibility visibility, QWidget *parent)
 MonthView::~MonthView()
 {
     if (calendar()) {
-        calendar()->unregisterObserver(d);
+        calendar()->unregisterObserver(d.get());
     }
-
-    delete d;
 }
 
 void MonthView::updateConfig()
@@ -283,7 +246,7 @@ KCalendarCore::DateList MonthView::selectedIncidenceDates() const
 {
     KCalendarCore::DateList list;
     if (d->scene->selectedItem()) {
-        IncidenceMonthItem *tmp = qobject_cast<IncidenceMonthItem *>(d->scene->selectedItem());
+        auto tmp = qobject_cast<IncidenceMonthItem *>(d->scene->selectedItem());
         if (tmp) {
             QDate selectedItemDate = tmp->realStartDate();
             if (selectedItemDate.isValid()) {
@@ -300,9 +263,9 @@ KCalendarCore::DateList MonthView::selectedIncidenceDates() const
 QDateTime MonthView::selectionStart() const
 {
     if (d->scene->selectedCell()) {
-        return QDateTime(d->scene->selectedCell()->date());
+        return QDateTime(d->scene->selectedCell()->date().startOfDay());
     } else {
-        return QDateTime();
+        return {};
     }
 }
 
@@ -333,16 +296,16 @@ bool MonthView::eventDurationHint(QDateTime &startDt, QDateTime &endDt, bool &al
 
 void MonthView::showIncidences(const Akonadi::Item::List &incidenceList, const QDate &date)
 {
-    Q_UNUSED(incidenceList);
-    Q_UNUSED(date);
+    Q_UNUSED(incidenceList)
+    Q_UNUSED(date)
 }
 
 void MonthView::changeIncidenceDisplay(const Akonadi::Item &incidence, int action)
 {
-    Q_UNUSED(incidence);
-    Q_UNUSED(action);
+    Q_UNUSED(incidence)
+    Q_UNUSED(action)
 
-    //TODO: add some more intelligence here...
+    // TODO: add some more intelligence here...
 
     // don't call reloadIncidences() directly. It would delete all
     // MonthItems, but this changeIncidenceDisplay()-method was probably
@@ -403,12 +366,10 @@ void MonthView::changeFullView()
 
     if (fullView) {
         d->fullView->setIcon(QIcon::fromTheme(QStringLiteral("view-restore")));
-        d->fullView->setToolTip(i18nc("@info:tooltip",
-                                      "Display calendar in a normal size"));
+        d->fullView->setToolTip(i18nc("@info:tooltip", "Display calendar in a normal size"));
     } else {
         d->fullView->setIcon(QIcon::fromTheme(QStringLiteral("view-fullscreen")));
-        d->fullView->setToolTip(i18nc("@info:tooltip",
-                                      "Display calendar in a full window"));
+        d->fullView->setToolTip(i18nc("@info:tooltip", "Display calendar in a full window"));
     }
     preferences()->setFullViewMonth(fullView);
     preferences()->writeConfig();
@@ -438,15 +399,16 @@ void MonthView::moveFwdMonth()
 
 void MonthView::showDates(const QDate &start, const QDate &end, const QDate &preferedMonth)
 {
-    Q_UNUSED(start);
-    Q_UNUSED(end);
-    Q_UNUSED(preferedMonth);
+    Q_UNUSED(start)
+    Q_UNUSED(end)
+    Q_UNUSED(preferedMonth)
     d->triggerDelayedReload(DatesChanged);
 }
 
 QPair<QDateTime, QDateTime> MonthView::actualDateRange(const QDateTime &start, const QDateTime &, const QDate &preferredMonth) const
 {
-    QDateTime dayOne = preferredMonth.isValid() ? QDateTime(preferredMonth) : start;
+    QDateTime dayOne = preferredMonth.isValid() ? QDateTime(preferredMonth.startOfDay()) : start;
+
     dayOne.setDate(QDate(dayOne.date().year(), dayOne.date().month(), 1));
     const int weekdayCol = (dayOne.date().dayOfWeek() + 7 - preferences()->firstDayOfWeek()) % 7;
     QDateTime actualStart = dayOne.addDays(-weekdayCol);
@@ -460,7 +422,7 @@ Akonadi::Item::List MonthView::selectedIncidences() const
 {
     Akonadi::Item::List selected;
     if (d->scene->selectedItem()) {
-        IncidenceMonthItem *tmp = qobject_cast<IncidenceMonthItem *>(d->scene->selectedItem());
+        auto tmp = qobject_cast<IncidenceMonthItem *>(d->scene->selectedItem());
         if (tmp) {
             Akonadi::Item incidenceSelected = tmp->akonadiItem();
             if (incidenceSelected.isValid()) {
@@ -471,6 +433,19 @@ Akonadi::Item::List MonthView::selectedIncidences() const
     return selected;
 }
 
+KHolidays::Holiday::List MonthView::holidays(QDate startDate, QDate endDate)
+{
+    KHolidays::Holiday::List holidays;
+    auto const regions = CalendarSupport::KCalPrefs::instance()->mHolidays;
+    for (auto const &r : regions) {
+        KHolidays::HolidayRegion region(r);
+        if (region.isValid()) {
+            holidays += region.rawHolidaysWithAstroSeasons(startDate, endDate);
+        }
+    }
+    return holidays;
+}
+
 void MonthView::reloadIncidences()
 {
     if (changes() == NothingChanged) {
@@ -481,7 +456,7 @@ void MonthView::reloadIncidences()
 
     MonthItem *itemToReselect = nullptr;
 
-    if (IncidenceMonthItem *tmp = qobject_cast<IncidenceMonthItem *>(d->scene->selectedItem())) {
+    if (auto tmp = qobject_cast<IncidenceMonthItem *>(d->scene->selectedItem())) {
         d->selectedItemId = tmp->akonadiItem().id();
         d->selectedItemDate = tmp->realStartDate();
         if (!d->selectedItemDate.isValid()) {
@@ -493,9 +468,8 @@ void MonthView::reloadIncidences()
     d->mBusyDays.clear();
     // build monthcells hash
     int i = 0;
-    for (QDate date = actualStartDateTime().date();
-         date <= actualEndDateTime().date(); date = date.addDays(1)) {
-        d->scene->mMonthCellMap[ date ] = new MonthCell(i, date, d->scene);
+    for (QDate date = actualStartDateTime().date(); date <= actualEndDateTime().date(); date = date.addDays(1)) {
+        d->scene->mMonthCellMap[date] = new MonthCell(i, date, d->scene);
         i++;
     }
 
@@ -507,12 +481,10 @@ void MonthView::reloadIncidences()
         occurIter.next();
 
         // Remove the two checks when filtering is done through a proxyModel, when using calendar search
-        if (!preferences()->showTodosMonthView()
-            && occurIter.incidence()->type() == KCalendarCore::Incidence::TypeTodo) {
+        if (!preferences()->showTodosMonthView() && occurIter.incidence()->type() == KCalendarCore::Incidence::TypeTodo) {
             continue;
         }
-        if (!preferences()->showJournalsMonthView()
-            && occurIter.incidence()->type() == KCalendarCore::Incidence::TypeJournal) {
+        if (!preferences()->showJournalsMonthView() && occurIter.incidence()->type() == KCalendarCore::Incidence::TypeJournal) {
             continue;
         }
 
@@ -528,14 +500,9 @@ void MonthView::reloadIncidences()
         }
         Q_ASSERT(item.isValid());
         Q_ASSERT(item.hasPayload());
-        MonthItem *manager = new IncidenceMonthItem(d->scene,
-                                                    calendar(),
-                                                    item,
-                                                    occurIter.incidence(),
-                                                    occurIter.occurrenceStartDate().toLocalTime().date());
+        MonthItem *manager = new IncidenceMonthItem(d->scene, calendar(), item, occurIter.incidence(), occurIter.occurrenceStartDate().toLocalTime().date());
         d->scene->mManagerList << manager;
-        if (d->selectedItemId == item.id()
-            && manager->realStartDate() == d->selectedItemDate) {
+        if (d->selectedItemId == item.id() && manager->realStartDate() == d->selectedItemDate) {
             // only select it outside the loop because we are still creating items
             itemToReselect = manager;
         }
@@ -546,33 +513,19 @@ void MonthView::reloadIncidences()
     }
 
     // add holidays
-    const QList<QDate> workDays = CalendarSupport::workDays(actualStartDateTime().date(),
-                                                            actualEndDateTime().date());
-
-    for (QDate date = actualStartDateTime().date();
-         date <= actualEndDateTime().date(); date = date.addDays(1)) {
-        // Only call CalendarSupport::holiday() if it's not a workDay, saves come cpu cicles.
-        if (!workDays.contains(date)) {
-            QStringList holidays(CalendarSupport::holiday(date));
-            if (!holidays.isEmpty()) {
-                MonthItem *holidayItem
-                    = new HolidayMonthItem(
-                          d->scene, date,
-                          holidays.join(i18nc("@item:intext delimiter for joining holiday names", ",")));
-                d->scene->mManagerList << holidayItem;
-            }
+    for (auto const &h : holidays(actualStartDateTime().date(), actualEndDateTime().date())) {
+        if (h.dayType() == KHolidays::Holiday::NonWorkday) {
+            MonthItem *holidayItem = new HolidayMonthItem(d->scene, h.observedStartDate(), h.observedEndDate(), h.name());
+            d->scene->mManagerList << holidayItem;
         }
     }
 
     // sort it
-    std::sort(d->scene->mManagerList.begin(),
-              d->scene->mManagerList.end(),
-              MonthItem::greaterThan);
+    std::sort(d->scene->mManagerList.begin(), d->scene->mManagerList.end(), MonthItem::greaterThan);
 
     // build each month's cell event list
-    for (MonthItem *manager : qAsConst(d->scene->mManagerList)) {
-        for (QDate date = manager->startDate();
-             date <= manager->endDate(); date = date.addDays(1)) {
+    for (MonthItem *manager : std::as_const(d->scene->mManagerList)) {
+        for (QDate date = manager->startDate(); date <= manager->endDate(); date = date.addDays(1)) {
             MonthCell *cell = d->scene->mMonthCellMap.value(date);
             if (cell) {
                 cell->mMonthItemList << manager;
@@ -580,12 +533,12 @@ void MonthView::reloadIncidences()
         }
     }
 
-    for (MonthItem *manager : qAsConst(d->scene->mManagerList)) {
+    for (MonthItem *manager : std::as_const(d->scene->mManagerList)) {
         manager->updateMonthGraphicsItems();
         manager->updatePosition();
     }
 
-    for (MonthItem *manager : qAsConst(d->scene->mManagerList)) {
+    for (MonthItem *manager : std::as_const(d->scene->mManagerList)) {
         manager->updateGeometry();
     }
 
@@ -602,8 +555,7 @@ void MonthView::calendarReset()
 
 QDate MonthView::averageDate() const
 {
-    return actualStartDateTime().date().addDays(
-        actualStartDateTime().date().daysTo(actualEndDateTime().date()) / 2);
+    return actualStartDateTime().date().addDays(actualStartDateTime().date().daysTo(actualEndDateTime().date()) / 2);
 }
 
 int MonthView::currentMonth() const
@@ -616,7 +568,7 @@ bool MonthView::usesFullWindow()
     return preferences()->fullViewMonth();
 }
 
-bool MonthView::isBusyDay(const QDate &day) const
+bool MonthView::isBusyDay(QDate day) const
 {
     return !d->mBusyDays[day].isEmpty();
 }
@@ -626,9 +578,9 @@ void MonthView::setCalendar(const Akonadi::ETMCalendar::Ptr &cal)
     Q_ASSERT(cal);
 
     if (calendar()) {
-        calendar()->unregisterObserver(d);
+        calendar()->unregisterObserver(d.get());
     }
 
     EventView::setCalendar(cal);
-    calendar()->registerObserver(d);
+    calendar()->registerObserver(d.get());
 }
