@@ -8,9 +8,9 @@
 
 #include "timelineitem.h"
 
-#include <KGantt/KGanttGlobal>
+#include <Akonadi/CalendarUtils>
 
-#include <CalendarSupport/Utils>
+#include <KGanttGlobal>
 
 #include <KCalUtils/IncidenceFormatter>
 
@@ -32,7 +32,7 @@ TimelineItem::TimelineItem(const Akonadi::ETMCalendar::Ptr &calendar, uint index
 
 void TimelineItem::insertIncidence(const Akonadi::Item &aitem, const QDateTime &_start, const QDateTime &_end)
 {
-    const Incidence::Ptr incidence = CalendarSupport::incidence(aitem);
+    const Incidence::Ptr incidence = Akonadi::CalendarUtils::incidence(aitem);
     QDateTime start(_start);
     QDateTime end(_end);
     if (!start.isValid()) {
@@ -102,14 +102,12 @@ TimelineSubItem::TimelineSubItem(const Akonadi::ETMCalendar::Ptr &calendar, cons
     , mToolTipNeedsUpdate(true)
 {
     setData(KGantt::TypeTask, KGantt::ItemTypeRole);
-    if (!CalendarSupport::incidence(incidence)->isReadOnly()) {
+    if (!Akonadi::CalendarUtils::incidence(incidence)->isReadOnly()) {
         setFlags(Qt::ItemIsSelectable);
     }
 }
 
-TimelineSubItem::~TimelineSubItem()
-{
-}
+TimelineSubItem::~TimelineSubItem() = default;
 
 void TimelineSubItem::setStartTime(const QDateTime &dt)
 {
@@ -139,8 +137,8 @@ void TimelineSubItem::updateToolTip()
 
     mToolTipNeedsUpdate = false;
 
-    setData(IncidenceFormatter::toolTipStr(CalendarSupport::displayName(mCalendar.data(), mIncidence.parentCollection()),
-                                           CalendarSupport::incidence(mIncidence),
+    setData(IncidenceFormatter::toolTipStr(Akonadi::CalendarUtils::displayName(mCalendar.data(), mIncidence.parentCollection()),
+                                           Akonadi::CalendarUtils::incidence(mIncidence),
                                            originalStart().date(),
                                            true),
             Qt::ToolTipRole);
