@@ -8,7 +8,7 @@
 
 #pragma once
 
-#include <Akonadi/Calendar/ETMCalendar>
+#include <Akonadi/ETMCalendar>
 #include <QStyledItemDelegate>
 
 class QPainter;
@@ -132,4 +132,3 @@ public:
 private:
     QTextDocument *m_textDoc = nullptr;
 };
-
