@@ -5,10 +5,8 @@
   SPDX-License-Identifier: GPL-2.0-or-later WITH Qt-Commercial-exception-1.0
 */
 
-#include "incidencetreemodel.h"
-#include "todomodel_p.h"
+#include "todomodel.h"
 
-#include <CalendarSupport/KCalPrefs>
 #include <CalendarSupport/Utils>
 #include <KCalUtils/IncidenceFormatter>
 #include <KCalendarCore/Attachment>
@@ -16,33 +14,34 @@
 #include <KEmailAddress>
 #include <KLocalizedString>
 
+#include <Akonadi/CalendarUtils>
+#include <Akonadi/IncidenceTreeModel>
+#include <Akonadi/TagCache>
+
 #include <KCalUtils/DndFactory>
 #include <KCalUtils/ICalDrag>
 #include <KCalUtils/VCalDrag>
 
 #include "calendarview_debug.h"
-#include <KMessageBox>
 
 #include <QIcon>
 #include <QMimeData>
 
-struct SourceModelIndex {
-    SourceModelIndex(int _r, int _c, void *_p, QAbstractItemModel *_m)
-        : r(_r)
-        , c(_c)
-        , p(_p)
-        , m(_m)
-    {
-    }
+class TodoModelPrivate
+{
+public:
+    TodoModelPrivate(const EventViews::PrefsPtr &preferences, TodoModel *qq);
 
-    operator QModelIndex()
-    {
-        return reinterpret_cast<QModelIndex &>(*this);
-    }
+    // TODO: O(N) complexity, see if the profiler complains about this
+    Akonadi::Item findItemByUid(const QString &uid, const QModelIndex &parent) const;
 
-    int r, c;
-    void *p;
-    const QAbstractItemModel *m = nullptr;
+    Akonadi::ETMCalendar::Ptr m_calendar;
+    Akonadi::IncidenceChanger *m_changer = nullptr;
+    EventViews::PrefsPtr m_preferences;
+
+    void onDataChanged(const QModelIndex &begin, const QModelIndex &end);
+
+    TodoModel *const q;
 };
 
 static bool isDueToday(const KCalendarCore::Todo::Ptr &todo)
@@ -51,8 +50,7 @@ static bool isDueToday(const KCalendarCore::Todo::Ptr &todo)
 }
 
 TodoModelPrivate::TodoModelPrivate(const EventViews::PrefsPtr &preferences, TodoModel *qq)
-    : QObject()
-    , m_preferences(preferences)
+    : m_preferences(preferences)
     , q(qq)
 {
 }
@@ -60,7 +58,7 @@ TodoModelPrivate::TodoModelPrivate(const EventViews::PrefsPtr &preferences, Todo
 Akonadi::Item TodoModelPrivate::findItemByUid(const QString &uid, const QModelIndex &parent) const
 {
     Q_ASSERT(!uid.isEmpty());
-    auto treeModel = qobject_cast<IncidenceTreeModel *>(q->sourceModel());
+    auto treeModel = qobject_cast<Akonadi::IncidenceTreeModel *>(q->sourceModel());
     if (treeModel) { // O(1) Shortcut
         return treeModel->item(uid);
     }
@@ -94,107 +92,8 @@ void TodoModelPrivate::onDataChanged(const QModelIndex &begin, const QModelIndex
     Q_EMIT q->dataChanged(proxyBegin, proxyEnd.sibling(proxyEnd.row(), TodoModel::ColumnCount - 1));
 }
 
-void TodoModelPrivate::onHeaderDataChanged(Qt::Orientation orientation, int first, int last)
-{
-    Q_EMIT q->headerDataChanged(orientation, first, last);
-}
-
-void TodoModelPrivate::onRowsAboutToBeInserted(const QModelIndex &parent, int begin, int end)
-{
-    const QModelIndex index = q->mapFromSource(parent);
-    Q_ASSERT(!(parent.isValid() ^ index.isValid())); // Both must be valid, or both invalid
-    Q_ASSERT(!(index.isValid() && index.model() != q));
-
-    q->beginInsertRows(index, begin, end);
-}
-
-void TodoModelPrivate::onRowsInserted(const QModelIndex &, int, int)
-{
-    q->endInsertRows();
-}
-
-void TodoModelPrivate::onRowsAboutToBeRemoved(const QModelIndex &parent, int begin, int end)
-{
-    const QModelIndex index = q->mapFromSource(parent);
-    Q_ASSERT(!(parent.isValid() ^ index.isValid())); // Both must be valid, or both invalid
-    Q_ASSERT(!(index.isValid() && index.model() != q));
-
-    q->beginRemoveRows(index, begin, end);
-}
-
-void TodoModelPrivate::onRowsRemoved(const QModelIndex &, int, int)
-{
-    q->endRemoveRows();
-}
-
-void TodoModelPrivate::onRowsAboutToBeMoved(const QModelIndex &sourceParent,
-                                            int sourceStart,
-                                            int sourceEnd,
-                                            const QModelIndex &destinationParent,
-                                            int destinationRow)
-{
-    Q_UNUSED(sourceParent)
-    Q_UNUSED(sourceStart)
-    Q_UNUSED(sourceEnd)
-    Q_UNUSED(destinationParent)
-    Q_UNUSED(destinationRow)
-    /* Disabled for now, layoutAboutToBeChanged() is emitted
-    q->beginMoveRows( q->mapFromSource( sourceParent ), sourceStart, sourceEnd,
-                      q->mapFromSource( destinationParent ), destinationRow );
-    */
-}
-
-void TodoModelPrivate::onRowsMoved(const QModelIndex &, int, int, const QModelIndex &, int)
-{
-    /*q->endMoveRows();*/
-}
-
-void TodoModelPrivate::onModelAboutToBeReset()
-{
-    q->beginResetModel();
-}
-
-void TodoModelPrivate::onModelReset()
-{
-    q->endResetModel();
-}
-
-void TodoModelPrivate::onLayoutAboutToBeChanged()
-{
-    Q_ASSERT(m_persistentIndexes.isEmpty());
-    Q_ASSERT(m_layoutChangePersistentIndexes.isEmpty());
-    Q_ASSERT(m_columns.isEmpty());
-    const QModelIndexList persistentIndexes = q->persistentIndexList();
-    for (const QModelIndex &persistentIndex : persistentIndexes) {
-        m_persistentIndexes << persistentIndex; // Stuff we have to update onLayoutChanged
-        Q_ASSERT(persistentIndex.isValid());
-        QModelIndex index_col0 = q->createIndex(persistentIndex.row(), 0, persistentIndex.internalPointer());
-        const QPersistentModelIndex srcPersistentIndex = q->mapToSource(index_col0);
-        Q_ASSERT(srcPersistentIndex.isValid());
-        m_layoutChangePersistentIndexes << srcPersistentIndex;
-        m_columns << persistentIndex.column();
-    }
-    Q_EMIT q->layoutAboutToBeChanged();
-}
-
-void TodoModelPrivate::onLayoutChanged()
-{
-    for (int i = 0; i < m_persistentIndexes.size(); ++i) {
-        QModelIndex newIndex_col0 = q->mapFromSource(m_layoutChangePersistentIndexes.at(i));
-        Q_ASSERT(newIndex_col0.isValid());
-        const int column = m_columns.at(i);
-        QModelIndex newIndex = column == 0 ? newIndex_col0 : q->createIndex(newIndex_col0.row(), column, newIndex_col0.internalPointer());
-        q->changePersistentIndex(m_persistentIndexes.at(i), newIndex);
-    }
-
-    m_layoutChangePersistentIndexes.clear();
-    m_persistentIndexes.clear();
-    m_columns.clear();
-    Q_EMIT q->layoutChanged();
-}
-
 TodoModel::TodoModel(const EventViews::PrefsPtr &preferences, QObject *parent)
-    : QAbstractProxyModel(parent)
+    : KExtraColumnsProxyModel(parent)
     , d(new TodoModelPrivate(preferences, this))
 {
     setObjectName(QStringLiteral("TodoModel"));
@@ -206,21 +105,21 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
 {
     Q_ASSERT(index.isValid());
     if (!index.isValid() || !d->m_calendar) {
-        return QVariant();
+        return {};
     }
 
     const QModelIndex sourceIndex = mapToSource(index.sibling(index.row(), 0));
     if (!sourceIndex.isValid()) {
-        return QVariant();
+        return {};
     }
     Q_ASSERT(sourceIndex.isValid());
     const auto item = sourceIndex.data(Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
     if (!item.isValid()) {
         qCWarning(CALENDARVIEW_LOG) << "Invalid index: " << sourceIndex;
         // Q_ASSERT( false );
-        return QVariant();
+        return {};
     }
-    const KCalendarCore::Todo::Ptr todo = CalendarSupport::todo(item);
+    const KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(item);
     if (!todo) {
         qCCritical(CALENDARVIEW_LOG) << "item.hasPayload()" << item.hasPayload();
         if (item.hasPayload<KCalendarCore::Incidence::Ptr>()) {
@@ -231,7 +130,7 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
         }
 
         Q_ASSERT(!"There's no to-do.");
-        return QVariant();
+        return {};
     }
 
     switch (role) {
@@ -264,7 +163,7 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
     case DescriptionRole:
         return todo->description();
     case CalendarRole:
-        return CalendarSupport::displayName(d->m_calendar.data(), item.parentCollection());
+        return Akonadi::CalendarUtils::displayName(d->m_calendar.data(), item.parentCollection());
     default:
         break; // column based model handling
     }
@@ -287,9 +186,9 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
             if (todo->priority() == 0) {
                 return QVariant(QStringLiteral("--"));
             }
-            return QVariant(todo->priority());
+            return {todo->priority()};
         case PercentColumn:
-            return QVariant(todo->percentComplete());
+            return {todo->percentComplete()};
         case StartDateColumn:
             return todo->hasStartDate() ? QLocale().toString(todo->dtStart().toLocalTime().date(), QLocale::ShortFormat) : QVariant(QString());
         case DueDateColumn:
@@ -303,9 +202,9 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
         case DescriptionColumn:
             return QVariant(todo->description());
         case CalendarColumn:
-            return QVariant(CalendarSupport::displayName(d->m_calendar.data(), item.parentCollection()));
+            return QVariant(Akonadi::CalendarUtils::displayName(d->m_calendar.data(), item.parentCollection()));
         }
-        return QVariant();
+        return {};
     }
 
     if (role == Qt::EditRole) {
@@ -313,11 +212,11 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
         case SummaryColumn:
             return QVariant(todo->summary());
         case RecurColumn:
-            return QVariant(todo->recurs());
+            return {todo->recurs()};
         case PriorityColumn:
-            return QVariant(todo->priority());
+            return {todo->priority()};
         case PercentColumn:
-            return QVariant(todo->percentComplete());
+            return {todo->percentComplete()};
         case StartDateColumn:
             return QVariant(todo->dtStart().date());
         case DueDateColumn:
@@ -329,18 +228,20 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
         case DescriptionColumn:
             return QVariant(todo->description());
         case CalendarColumn:
-            return QVariant(CalendarSupport::displayName(d->m_calendar.data(), item.parentCollection()));
+            return QVariant(Akonadi::CalendarUtils::displayName(d->m_calendar.data(), item.parentCollection()));
         }
-        return QVariant();
+        return {};
     }
 
     // set the tooltip for every item
     if (role == Qt::ToolTipRole) {
         if (d->m_preferences->enableToolTips()) {
-            return QVariant(
-                KCalUtils::IncidenceFormatter::toolTipStr(CalendarSupport::displayName(d->m_calendar.data(), item.parentCollection()), todo, QDate(), true));
+            return QVariant(KCalUtils::IncidenceFormatter::toolTipStr(Akonadi::CalendarUtils::displayName(d->m_calendar.data(), item.parentCollection()),
+                                                                      todo,
+                                                                      QDate(),
+                                                                      true));
         } else {
-            return QVariant();
+            return {};
         }
     }
 
@@ -356,13 +257,13 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
     // indicate if a row is checked (=completed) only in the first column
     if (role == Qt::CheckStateRole && index.column() == 0) {
         if (hasChildren(index) && !index.parent().isValid()) {
-            return QVariant();
+            return {};
         }
 
         if (todo->isCompleted()) {
-            return QVariant(Qt::Checked);
+            return {Qt::Checked};
         } else {
-            return QVariant(Qt::Unchecked);
+            return {Qt::Unchecked};
         }
     }
 
@@ -375,9 +276,9 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
             return QVariant(QIcon::fromTheme(QStringLiteral("task-recurring")));
         }
         const QStringList categories = todo->categories();
-        return categories.isEmpty() ? QVariant() : QVariant(CalendarSupport::KCalPrefs::instance()->categoryColor(categories.first()));
+        return categories.isEmpty() ? QVariant() : QVariant(Akonadi::TagCache::instance()->tagColor(categories.first()));
     } else if (role == Qt::DecorationRole) {
-        return QVariant();
+        return {};
     }
 
     if (role == TodoRole) {
@@ -390,11 +291,11 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
 
     if (role == IsRichTextRole) {
         if (index.column() == SummaryColumn) {
-            return QVariant(todo->summaryIsRich());
+            return {todo->summaryIsRich()};
         } else if (index.column() == DescriptionColumn) {
-            return QVariant(todo->descriptionIsRich());
+            return {todo->descriptionIsRich()};
         } else {
-            return QVariant();
+            return {};
         }
     }
 
@@ -408,16 +309,26 @@ QVariant TodoModel::data(const QModelIndex &index, int role) const
         case DueDateColumn:
         case CategoriesColumn:
         case CalendarColumn:
-            return QVariant(Qt::AlignHCenter | Qt::AlignVCenter);
+            return {Qt::AlignHCenter | Qt::AlignVCenter};
         }
-        return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
+        return {Qt::AlignLeft | Qt::AlignVCenter};
     }
 
     if (sourceModel()) {
         return sourceModel()->data(mapToSource(index.sibling(index.row(), 0)), role);
     }
 
-    return QVariant();
+    return {};
+}
+
+QVariant TodoModel::extraColumnData(const QModelIndex &parent, int row, int extraColumn, int role) const
+{
+    // we customize all columns, not just the extra ones, and thus do all that in ::data()
+    Q_UNUSED(parent);
+    Q_UNUSED(row);
+    Q_UNUSED(extraColumn);
+    Q_UNUSED(role);
+    return {};
 }
 
 bool TodoModel::setData(const QModelIndex &index, const QVariant &value, int role)
@@ -435,7 +346,7 @@ bool TodoModel::setData(const QModelIndex &index, const QVariant &value, int rol
     }
 
     const auto item = data(index, Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
-    const KCalendarCore::Todo::Ptr todo = CalendarSupport::todo(item);
+    const KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(item);
 
     if (!item.isValid() || !todo) {
         qCWarning(CALENDARVIEW_LOG) << "TodoModel::setData() called, bug item is invalid or doesn't have payload";
@@ -507,19 +418,6 @@ bool TodoModel::setData(const QModelIndex &index, const QVariant &value, int rol
     }
 }
 
-int TodoModel::rowCount(const QModelIndex &parent) const
-{
-    if (sourceModel()) {
-        if (parent.isValid()) {
-            QModelIndex parent_col0 = createIndex(parent.row(), 0, parent.internalPointer());
-            return sourceModel()->rowCount(mapToSource(parent_col0));
-        } else {
-            return sourceModel()->rowCount();
-        }
-    }
-    return 0;
-}
-
 int TodoModel::columnCount(const QModelIndex &) const
 {
     return ColumnCount;
@@ -534,60 +432,15 @@ void TodoModel::setSourceModel(QAbstractItemModel *model)
     beginResetModel();
 
     if (sourceModel()) {
-        disconnect(sourceModel(), SIGNAL(dataChanged(QModelIndex, QModelIndex)), d.get(), SLOT(onDataChanged(QModelIndex, QModelIndex)));
-        disconnect(sourceModel(), SIGNAL(headerDataChanged(Qt::Orientation, int, int)), d.get(), SLOT(onHeaderDataChanged(Qt::Orientation, int, int)));
-
-        disconnect(sourceModel(), SIGNAL(rowsInserted(QModelIndex, int, int)), d.get(), SLOT(onRowsInserted(QModelIndex, int, int)));
-
-        disconnect(sourceModel(), SIGNAL(rowsRemoved(QModelIndex, int, int)), d.get(), SLOT(onRowsRemoved(QModelIndex, int, int)));
-
-        disconnect(sourceModel(),
-                   SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)),
-                   d.get(),
-                   SLOT(onRowsMoved(QModelIndex, int, int, QModelIndex, int)));
-
-        disconnect(sourceModel(), SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)), d.get(), SLOT(onRowsAboutToBeInserted(QModelIndex, int, int)));
-
-        disconnect(sourceModel(), SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), d.get(), SLOT(onRowsAboutToBeRemoved(QModelIndex, int, int)));
-
-        disconnect(sourceModel(), SIGNAL(modelAboutToBeReset()), d.get(), SLOT(onModelAboutToBeReset()));
-
-        disconnect(sourceModel(), SIGNAL(modelReset()), d.get(), SLOT(onModelReset()));
-
-        disconnect(sourceModel(), SIGNAL(layoutAboutToBeChanged()), d.get(), SLOT(onLayoutAboutToBeChanged()));
-
-        disconnect(sourceModel(), SIGNAL(layoutChanged()), d.get(), SLOT(onLayoutChanged()));
+        disconnect(sourceModel(), &QAbstractItemModel::dataChanged, this, nullptr);
     }
 
-    QAbstractProxyModel::setSourceModel(model);
+    KExtraColumnsProxyModel::setSourceModel(model);
 
     if (sourceModel()) {
-        connect(sourceModel(), SIGNAL(dataChanged(QModelIndex, QModelIndex)), d.get(), SLOT(onDataChanged(QModelIndex, QModelIndex)));
-
-        connect(sourceModel(), SIGNAL(headerDataChanged(Qt::Orientation, int, int)), d.get(), SLOT(onHeaderDataChanged(Qt::Orientation, int, int)));
-
-        connect(sourceModel(), SIGNAL(rowsAboutToBeInserted(QModelIndex, int, int)), d.get(), SLOT(onRowsAboutToBeInserted(QModelIndex, int, int)));
-
-        connect(sourceModel(), SIGNAL(rowsInserted(QModelIndex, int, int)), d.get(), SLOT(onRowsInserted(QModelIndex, int, int)));
-
-        connect(sourceModel(), SIGNAL(rowsAboutToBeRemoved(QModelIndex, int, int)), d.get(), SLOT(onRowsAboutToBeRemoved(QModelIndex, int, int)));
-
-        connect(sourceModel(), SIGNAL(rowsRemoved(QModelIndex, int, int)), d.get(), SLOT(onRowsRemoved(QModelIndex, int, int)));
-
-        connect(sourceModel(),
-                SIGNAL(rowsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)),
-                d.get(),
-                SLOT(onRowsAboutToBeMoved(QModelIndex, int, int, QModelIndex, int)));
-
-        connect(sourceModel(), SIGNAL(rowsMoved(QModelIndex, int, int, QModelIndex, int)), d.get(), SLOT(onRowsMoved(QModelIndex, int, int, QModelIndex, int)));
-
-        connect(sourceModel(), SIGNAL(modelAboutToBeReset()), d.get(), SLOT(onModelAboutToBeReset()));
-
-        connect(sourceModel(), SIGNAL(modelReset()), d.get(), SLOT(onModelReset()));
-
-        connect(sourceModel(), SIGNAL(layoutAboutToBeChanged()), d.get(), SLOT(onLayoutAboutToBeChanged()));
-
-        connect(sourceModel(), SIGNAL(layoutChanged()), d.get(), SLOT(onLayoutChanged()));
+        connect(sourceModel(), &QAbstractItemModel::dataChanged, this, [this](const auto &begin, const auto &end) {
+            d->onDataChanged(begin, end);
+        });
     }
 
     endResetModel();
@@ -601,7 +454,7 @@ void TodoModel::setIncidenceChanger(Akonadi::IncidenceChanger *changer)
 QVariant TodoModel::headerData(int column, Qt::Orientation orientation, int role) const
 {
     if (orientation != Qt::Horizontal) {
-        return QVariant();
+        return {};
     }
 
     if (role == Qt::DisplayRole) {
@@ -639,11 +492,11 @@ QVariant TodoModel::headerData(int column, Qt::Orientation orientation, int role
         case DueDateColumn:
         case CategoriesColumn:
         case CalendarColumn:
-            return QVariant(Qt::AlignHCenter);
+            return {Qt::AlignHCenter};
         }
-        return QVariant();
+        return {};
     }
-    return QVariant();
+    return {};
 }
 
 void TodoModel::setCalendar(const Akonadi::ETMCalendar::Ptr &calendar)
@@ -697,26 +550,23 @@ bool TodoModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
             // we don't want to change the created todo, but the one which is already
             // stored in our calendar / tree
             const Akonadi::Item item = d->findItemByUid(t->uid(), QModelIndex());
-            KCalendarCore::Todo::Ptr todo = CalendarSupport::todo(item);
+            KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(item);
             KCalendarCore::Todo::Ptr destTodo;
             if (parent.isValid()) {
                 const auto parentItem = this->data(parent, Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
                 if (parentItem.isValid()) {
-                    destTodo = CalendarSupport::todo(parentItem);
+                    destTodo = Akonadi::CalendarUtils::todo(parentItem);
                 }
             }
 
             KCalendarCore::Incidence::Ptr tmp = destTodo;
             while (tmp) {
                 if (tmp->uid() == todo->uid()) { // correct, don't use instanceIdentifier() here
-                    KMessageBox::information(nullptr,
-                                             i18n("Cannot move to-do to itself or a child of itself."),
-                                             i18n("Drop To-do"),
-                                             QStringLiteral("NoDropTodoOntoItself"));
+                    Q_EMIT dropOnSelfRejected();
                     return false;
                 }
                 const QString parentUid = tmp->relatedTo();
-                tmp = CalendarSupport::incidence(d->m_calendar->item(parentUid));
+                tmp = Akonadi::CalendarUtils::incidence(d->m_calendar->item(parentUid));
             }
 
             if (!destTodo || !destTodo->hasRecurrenceId()) {
@@ -741,7 +591,7 @@ bool TodoModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int r
             }
 
             const auto parentItem = this->data(parent, Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
-            KCalendarCore::Todo::Ptr destTodo = CalendarSupport::todo(parentItem);
+            KCalendarCore::Todo::Ptr destTodo = Akonadi::CalendarUtils::todo(parentItem);
 
             if (data->hasText()) {
                 QString text = data->text();
@@ -776,7 +626,7 @@ Qt::ItemFlags TodoModel::flags(const QModelIndex &index) const
         return Qt::NoItemFlags;
     }
 
-    Qt::ItemFlags ret = QAbstractItemModel::flags(index);
+    Qt::ItemFlags ret = KExtraColumnsProxyModel::flags(index);
 
     const auto item = data(index, Akonadi::EntityTreeModel::ItemRole).value<Akonadi::Item>();
 
@@ -789,7 +639,7 @@ Qt::ItemFlags TodoModel::flags(const QModelIndex &index) const
 
     ret |= Qt::ItemIsDragEnabled;
 
-    const KCalendarCore::Todo::Ptr todo = CalendarSupport::todo(item);
+    const KCalendarCore::Todo::Ptr todo = Akonadi::CalendarUtils::todo(item);
 
     if (d->m_calendar->hasRight(item, Akonadi::Collection::CanChangeItem)) {
         // the following columns are editable:
@@ -819,82 +669,6 @@ Qt::ItemFlags TodoModel::flags(const QModelIndex &index) const
     return ret;
 }
 
-QModelIndex TodoModel::mapFromSource(const QModelIndex &sourceIndex) const
-{
-    if (!sourceModel() || !sourceIndex.isValid()) {
-        return {};
-    }
-
-    Q_ASSERT(sourceIndex.internalPointer());
-
-    return createIndex(sourceIndex.row(), 0, sourceIndex.internalPointer());
-}
-
-QModelIndex TodoModel::mapToSource(const QModelIndex &proxyIndex) const
-{
-    if (!sourceModel() || !proxyIndex.isValid()) {
-        return {};
-    }
-
-    if (proxyIndex.column() != 0) {
-        qCCritical(CALENDARVIEW_LOG) << "Map to source called with column>0, but source model only has 1 column";
-        Q_ASSERT(false);
-    }
-
-    Q_ASSERT(proxyIndex.internalPointer());
-
-    // we convert to column 0
-    const QModelIndex sourceIndex = SourceModelIndex(proxyIndex.row(), 0, proxyIndex.internalPointer(), sourceModel());
-
-    return sourceIndex;
-}
-
-QModelIndex TodoModel::index(int row, int column, const QModelIndex &parent) const
-{
-    if (!sourceModel()) {
-        return QModelIndex();
-    }
-
-    Q_ASSERT(!parent.isValid() || parent.internalPointer());
-    QModelIndex parent_col0 = parent.isValid() ? createIndex(parent.row(), 0, parent.internalPointer()) : QModelIndex();
-
-    // Lets preserve the original internalPointer
-    const QModelIndex index = mapFromSource(sourceModel()->index(row, 0, mapToSource(parent_col0)));
-
-    Q_ASSERT(!index.isValid() || index.internalPointer());
-
-    if (index.isValid()) {
-        return createIndex(row, column, index.internalPointer());
-    }
-
-    return {};
-}
-
-QModelIndex TodoModel::parent(const QModelIndex &child) const
-{
-    if (!sourceModel() || !child.isValid()) {
-        return QModelIndex();
-    }
-
-    Q_ASSERT(child.internalPointer());
-    const QModelIndex child_col0 = createIndex(child.row(), 0, child.internalPointer());
-    QModelIndex parentIndex = mapFromSource(sourceModel()->parent(mapToSource(child_col0)));
-
-    Q_ASSERT(!parentIndex.isValid() || parentIndex.internalPointer());
-    if (parentIndex.isValid()) { // preserve original column
-        return createIndex(parentIndex.row(), child.column(), parentIndex.internalPointer());
-    }
-
-    return {};
-}
-
-QModelIndex TodoModel::buddy(const QModelIndex &index) const
-{
-    // We reimplement because the default implementation calls mapToSource() and
-    // source model doesn't have the same number of columns.
-    return index;
-}
-
 QHash<int, QByteArray> TodoModel::roleNames() const
 {
     return {
