@@ -7,20 +7,26 @@
 
 #pragma once
 
+#include "eventviews_export.h"
 #include "prefs.h"
 
-#include <Akonadi/Calendar/ETMCalendar>
-#include <Akonadi/Calendar/IncidenceChanger>
-#include <Item>
+#include <Akonadi/ETMCalendar>
+#include <Akonadi/IncidenceChanger>
+#include <Akonadi/Item>
 
-#include <EntityTreeModel>
+#include <Akonadi/EntityTreeModel>
 #include <KCalendarCore/Todo>
-#include <QAbstractItemModel>
-#include <QAbstractProxyModel>
+
+#include <KExtraColumnsProxyModel>
+
+#include <memory>
 
 class QMimeData;
 
-class TodoModel : public QAbstractProxyModel
+class TodoModelPrivate;
+
+/** Expands an IncidenceTreeModel by additional columns for showing todos. */
+class EVENTVIEWS_EXPORT TodoModel : public KExtraColumnsProxyModel
 {
     Q_OBJECT
 
@@ -36,23 +42,36 @@ public:
         CategoriesColumn,
         DescriptionColumn,
         CalendarColumn,
+        CompletedDateColumn,
         ColumnCount // Just for iteration/column count purposes. Always keep at the end of enum.
     };
 
     /** This enum defines the user defined roles of the items in this model */
-    enum { TodoRole = Akonadi::EntityTreeModel::UserRole + 1, IsRichTextRole, TodoPtrRole };
+    enum {
+        TodoRole = Akonadi::EntityTreeModel::UserRole + 1,
+        TodoPtrRole,
+        IsRichTextRole,
+        SummaryRole,
+        RecurRole,
+        PriorityRole,
+        PercentRole,
+        StartDateRole,
+        DueDateRole,
+        CategoriesRole,
+        DescriptionRole,
+        CalendarRole,
+    };
 
     explicit TodoModel(const EventViews::PrefsPtr &preferences, QObject *parent = nullptr);
 
     ~TodoModel() override;
 
-    Q_REQUIRED_RESULT int rowCount(const QModelIndex &parent = QModelIndex()) const override;
-
     Q_REQUIRED_RESULT int columnCount(const QModelIndex &parent = QModelIndex()) const override;
 
     void setSourceModel(QAbstractItemModel *sourceModel) override;
 
     Q_REQUIRED_RESULT QVariant data(const QModelIndex &index, int role) const override;
+    QVariant extraColumnData(const QModelIndex &parent, int row, int extraColumn, int role = Qt::DisplayRole) const override;
 
     Q_REQUIRED_RESULT bool setData(const QModelIndex &index, const QVariant &value, int role) override;
 
@@ -72,18 +91,15 @@ public:
 
     Q_REQUIRED_RESULT Qt::ItemFlags flags(const QModelIndex &index) const override;
 
-    Q_REQUIRED_RESULT QModelIndex parent(const QModelIndex &child) const override;
+    Q_REQUIRED_RESULT QHash<int, QByteArray> roleNames() const override;
 
-    Q_REQUIRED_RESULT QModelIndex mapFromSource(const QModelIndex &sourceIndex) const override;
-
-    Q_REQUIRED_RESULT QModelIndex mapToSource(const QModelIndex &proxyIndex) const override;
-
-    Q_REQUIRED_RESULT QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
-
-    Q_REQUIRED_RESULT QModelIndex buddy(const QModelIndex &index) const override;
+Q_SIGNALS:
+    /** Emitted when dropMimeData() rejected a drop
+     *  on the same item or any of its children.
+     */
+    void dropOnSelfRejected();
 
 private:
-    class Private;
-    Private *const d;
+    friend class TodoModelPrivate;
+    std::unique_ptr<TodoModelPrivate> const d;
 };
-
