@@ -1,49 +1,38 @@
 /*
   This file is part of KOrganizer.
 
-  Copyright (c) 2001 Cornelius Schumacher <schumacher@kde.org>
+  SPDX-FileCopyrightText: 2001 Cornelius Schumacher <schumacher@kde.org>
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-
-  As a special exception, permission is given to link this program
-  with any edition of Qt, and distribute the resulting executable,
-  without including the source code for Qt in the source distribution.
+  SPDX-License-Identifier: GPL-2.0-or-later WITH Qt-Commercial-exception-1.0
 */
-#ifndef CALENDARVIEWS_WHATSNEXTVIEW_H
-#define CALENDARVIEWS_WHATSNEXTVIEW_H
+#pragma once
 
 #include "eventview.h"
 
-#include <Akonadi/Calendar/IncidenceChanger>
+#include <Akonadi/IncidenceChanger>
 
 #include <KIconLoader>
 
-#include <QUrl>
 #include <QTextBrowser>
+#include <QUrl>
 
-namespace EventViews {
+namespace EventViews
+{
 class WhatsNextTextBrowser : public QTextBrowser
 {
     Q_OBJECT
 public:
-    explicit WhatsNextTextBrowser(QWidget *parent) : QTextBrowser(parent)
+    explicit WhatsNextTextBrowser(QWidget *parent)
+        : QTextBrowser(parent)
     {
     }
 
     /** Reimplemented from QTextBrowser to handle links. */
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     void setSource(const QUrl &name) override;
+#else
+    void doSetSource(const QUrl &name, QTextDocument::ResourceType type = QTextDocument::UnknownResource) override;
+#endif
 
 Q_SIGNALS:
     void showIncidence(const QString &uid);
@@ -62,12 +51,12 @@ public:
     Q_REQUIRED_RESULT int currentDateCount() const override;
     Q_REQUIRED_RESULT Akonadi::Item::List selectedIncidences() const override
     {
-        return Akonadi::Item::List();
+        return {};
     }
 
     Q_REQUIRED_RESULT KCalendarCore::DateList selectedIncidenceDates() const override
     {
-        return KCalendarCore::DateList();
+        return {};
     }
 
     Q_REQUIRED_RESULT bool supportsDateNavigation() const
@@ -99,5 +88,3 @@ private:
     Akonadi::Item::List mTodos;
 };
 }
-
-#endif
